/*
 *	wave.h
 *
 *  Copyright (C) 1998 Hermann Samso
 */


#ifndef WAVE_H
#define WAVE_H

#include "types.h"

#define MAXWAVES 256          /* size of waves pool */

typedef struct SAMPLE {
	FLOAT left;
	FLOAT right;
} SAMPLE;


typedef struct WAVE {
	CHAR name[42];
	ULONG size;			// in SAMPLEs
	FLOAT peak;			// biggest amplitude
	struct SAMPLE* samples;	// SAMPLE buffer

/* 'buffer' should move to a CHANNEL or BUS module, see TODO file */

	struct SAMPLE* buffer;		// subset of 'samples' with size 'audiobufsize'

/* The following should move to a BREAK struct, see TODO file */

	ULONG start;		// first sample
	ULONG position;		// actual playing position
	BOOL active;
	INT loop;			// loop how many times?
	INT loop_counter;
	struct WAVE* next;			// next wave to play
	struct WAVE* previous;		// previous wave played
	FLOAT lvol;			// volumes
	FLOAT rvol;
	FLOAT lpan;			// pan
	FLOAT rpan;

} WAVE;

BOOL left_up_wrap;
BOOL left_down_wrap;
BOOL right_up_wrap;
BOOL right_down_wrap;

WAVE* WaveLoadRaw(CHAR* fname, int channels);
void WaveSaveRaw(CHAR* fname, WAVE* wave);
WAVE* WaveNew(CHAR* name, ULONG size);
WAVE* WaveCopy(WAVE* dest, WAVE* src);
WAVE* WaveClone(WAVE* wave);
ULONG WaveBlockToWave(WAVE* dest, WAVE* src, ULONG src_start, BOOL wraparound);
ULONG WaveBlockToSample(SAMPLE* dest, WAVE* src, ULONG src_start, ULONG size, BOOL wraparound);
BOOL WaveRegister(WAVE* wave);	//returns TRUE if successful
void WaveClear(WAVE* wave);
void WaveDelete(void);
void WaveFree(WAVE* wave);
WAVE* WaveGet(void);
WAVE* WaveNext(void);
WAVE* WavePrevious(void);
void WaveSelectByName(CHAR* name);
void WaveSelectByIndex(INT number);
WAVE* WaveGetByName(CHAR* name);
WAVE* WaveGetByIndex(INT number);
INT WaveGetIndex(void);			//returns index of selected WAVE, -1 if none selected
void WaveDisplay(void);
void WaveClearDisplay(void);
SWORD* WavePlay(WAVE* wave);
void WavePlaySelected(void);
INT WaveHowMany(void);
void WaveToRaw(SWORD* raw, WAVE* wave, ULONG start, ULONG size);
void WaveNormalize(void);
void WaveScale(FLOAT amp);
FLOAT WavePeak(void);
void WaveResize(ULONG size);
void WaveRoll(SLONG size);
#endif
